'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.winSpawn = exports.spawnShell = undefined;

var _child_process = require('child_process');

var _defaultShell = require('default-shell');

var _defaultShell2 = _interopRequireDefault(_defaultShell);

var _os = require('os');

var _net = require('net');

var _net2 = _interopRequireDefault(_net);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodePty = require('node-pty');

var pty = _interopRequireWildcard(_nodePty);

var _tmp2 = require('tmp');

var _tmp3 = _interopRequireDefault(_tmp2);

var _promisifyNode = require('promisify-node');

var _promisifyNode2 = _interopRequireDefault(_promisifyNode);

var _constants = require('../constants');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var tmp = (0, _promisifyNode2.default)(_tmp3.default);
// Cleanup even when there are errors
tmp.setGracefulCleanup();

var IS_WINDOWS = process.platform === 'win32';

var buildSocketPath = function buildSocketPath() {
  return tmp.dir().then(function (tmpPath) {
    var prefix = IS_WINDOWS ? '\\\\.\\pipe' : '';
    var cleanedPath = IS_WINDOWS ? tmpPath.replace('C:\\', '') : tmpPath;
    cleanedPath = cleanedPath.replace('.tmp', '');
    return _path2.default.join(prefix, cleanedPath, 'echo.sock');
  });
};

var buildCredentialsCallbackProcess = function buildCredentialsCallbackProcess(spawnedProcess, callback, reject) {
  var credentials = {};
  var credentialsCallback = function credentialsCallback(forSsh) {
    return function (username, password, cancel) {
      if (cancel) {
        // we are done here
        spawnedProcess.destroy();
        return reject(new Error('LFS action cancelled'));
      }

      credentials = { username: username, password: password };
      spawnedProcess.write(forSsh ? credentials.username : credentials.password);
      spawnedProcess.write(_os.EOL);
    };
  };

  return function (chunk) {
    var output = chunk.toString();

    if (output.match(_constants.regex.USERNAME)) {
      if (credentials.username) {
        spawnedProcess.write(credentials.username);
        spawnedProcess.write(_os.EOL);
      } else {
        // We got a username so we must not be ssh
        callback(false, credentialsCallback(false));
      }
    } else if (output.match(_constants.regex.PASSWORD)) {
      if (credentials.password) {
        var password = credentials.password || _os.EOL;
        spawnedProcess.write(password);
        spawnedProcess.write(_os.EOL);
      } else {
        // no username so ssh
        callback(true, credentialsCallback(true));
      }
    }
    return output;
  };
};

var buildSocket = function buildSocket(size, closeProcess, socketName, mainResolve, mainReject) {
  return new Promise(function (resolve, reject) {
    var bufferList = [];
    var currentSize = 0;
    var resolved = false;

    var closedOrEnded = function closedOrEnded() {
      // For some reason this fires twice so after
      // the first fire let's move on
      if (!resolved) {
        mainResolve({ stdout: Buffer.concat(bufferList) });
        resolved = true;
        closeProcess();
      }
    };

    var socketServer = _net2.default.createServer(function (socket) {
      socket.on('end', closedOrEnded);
      socket.on('close', closedOrEnded);
      socket.on('data', function (data) {
        currentSize += data.length;
        bufferList.push(data);
        if (currentSize === size) {
          socketServer.close(function () {});
        }
      });
      socket.on('error', mainReject);
    });
    socketServer.listen(socketName);
    socketServer.on('listening', function () {
      resolve(socketName);
    });
    socketServer.on('error', reject);
    socketServer.on('close', closedOrEnded);
  });
};

var spawnShell = exports.spawnShell = function spawnShell(command, opts, size, callback) {
  return new Promise(function (resolve, reject) {
    var spawnedProcess = void 0;
    var destroyProcess = function destroyProcess() {
      return spawnedProcess.destroy();
    };
    return buildSocketPath().then(function (socket) {
      return buildSocket(size, destroyProcess, socket, resolve, reject);
    }).then(function (socketName) {
      var options = _ramda2.default.mergeDeepRight(opts, { env: process.env, encoding: null });

      spawnedProcess = pty.spawn(_defaultShell2.default, [], options);

      var processChunk = callback && typeof callback === 'function' ? buildCredentialsCallbackProcess(spawnedProcess, callback, reject) : function (chunk) {
        return chunk.toString();
      };

      var commandSuffix = IS_WINDOWS ? ' >' + socketName : ' | nc -U ' + socketName;

      spawnedProcess.write('' + command + commandSuffix);
      spawnedProcess.write(_os.EOL);

      spawnedProcess.on('data', function (data) {
        processChunk(data);
      });

      spawnedProcess.on('error', reject);
    }).catch(reject);
  });
};

var winSpawn = exports.winSpawn = function winSpawn(command, input, opts) {
  return new Promise(function (resolve, reject) {
    var options = _ramda2.default.mergeDeepRight(opts, { env: process.env, shell: true });

    var argList = command.trim().split(' ');
    var cmd = argList.shift();
    var args = argList;

    var spawnedProcess = (0, _child_process.spawn)(cmd, args, options);

    var bufferList = [];
    spawnedProcess.stdout.on('data', function (data) {
      bufferList.push(data);
    });

    var closeOrExit = function closeOrExit() {
      var code = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      return resolve({
        code: code,
        stdout: Buffer.concat(bufferList)
      });
    };

    spawnedProcess.on('close', closeOrExit);
    spawnedProcess.on('exit', closeOrExit);
    spawnedProcess.stderr.on('data', function (data) {
      reject(new Error(data.toString()));
    });

    spawnedProcess.stdin.write(input);
    spawnedProcess.stdin.write(_os.EOL);
  });
};

var spawn = function spawn(command, opts, callback) {
  return new Promise(function (resolve, reject) {
    var options = _ramda2.default.mergeDeepRight(opts, { env: process.env });

    var argList = command.trim().split(' ');
    var cmd = argList.shift() + (IS_WINDOWS ? '.exe' : '');
    var args = argList;

    var stdout = '';
    var spawnedProcess = pty.spawn(cmd, args, options);

    var processChunk = callback && typeof callback === 'function' ? buildCredentialsCallbackProcess(spawnedProcess, callback, reject) : function (chunk) {
      return chunk.toString();
    };

    spawnedProcess.on('data', function (data) {
      stdout += processChunk(data);
    });

    var closeOrExit = function closeOrExit() {
      var code = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      return resolve({
        code: code,
        stdout: stdout
      });
    };

    spawnedProcess.on('close', closeOrExit);
    spawnedProcess.on('exit', closeOrExit);
    spawnedProcess.on('error', reject);
  });
};

exports.default = spawn;